@@ -1,10 +1,12 @@
+import asyncio
 import enum
 import re
 
-import requests
+import aiohttp
 
 from json.decoder import JSONDecodeError
 
+from aiohttp import ClientTimeout
 from graphql import (
     GraphQLNonNull,
     GraphQLList,
@@ -126,11 +128,15 @@ def url_to_ast(
 ) -> GraphQLSchema:
     _introspect_query = get_introspection_query()
 
-    response = http_query(url=url,
-                          query=_introspect_query,
-                          http_method=http_method,
-                          http_headers=http_headers,
-                          verify=verify)
+    response = asyncio.run(
+        http_query(
+            url=url,
+            query=_introspect_query,
+            http_method=http_method,
+            http_headers=http_headers,
+            verify=verify
+        )
+    )
     errors = response.get('errors')
 
     if errors:
@@ -148,7 +154,7 @@ def executor_to_ast(executor) -> GraphQLSchema:
     return build_client_schema(introspect_schema)
 
 
-def http_query(
+async def http_query(
     url,
     query,
     variable_values=None,
@@ -169,34 +175,36 @@ def http_query(
     if operation_name:
         params["operationName"] = operation_name
 
+    session = aiohttp.ClientSession()
+
     if http_method == "GET":
-        r = requests.get(
+        r = await session.get(
             url,
             params=params,
-            verify=verify,
+            ssl=verify,
             headers={'Accept': 'application/json', **http_headers},
-            timeout=http_timeout
+            timeout=ClientTimeout(total=http_timeout)
         )
 
     elif http_method == "POST":
-        r = requests.post(
+        r = await session.post(
             url,
             json=params,
-            verify=verify,
+            ssl=verify,
             headers={'Accept': 'application/json', **http_headers},
-            timeout=http_timeout
+            timeout=ClientTimeout(total=http_timeout)
         )
 
     else:
         raise AttributeError(f"Invalid HTTP method {http_method}")
 
-    if r.status_code != 200:
+    if r.status != 200:
         raise ValueError(
-            f"Invalid response code '{r.status_code}'"
+            f"Invalid response code '{r.status}'"
         )
 
     try:
-        json = r.json()
+        json = await r.json()
     except JSONDecodeError as e:
         raise ValueError(
             f"{e}, unable to decode JSON"
